% Function for augmentig system model with an LPF state

function model = augmentWithLPF(modelIn, b, dt, inputs)
Tlp = dt*((1-b)/b); % LPF time constant [min]

NstatesIn = size(modelIn.F, 1); 
Nstates = NstatesIn+1;              

model.F = modelIn.F;
model.F(Nstates,Nstates) = -1/Tlp;
model.F(Nstates,Nstates-1) = 1/Tlp;  % System matrix (continuous)

model.Phi=expm(model.F*dt);          % System matrix (discrete)

% Unknown input transition matrix (continous/discrete)
if inputs
    model.B = modelIn.B;
    model.B(Nstates,1) = 0;
    model.Eps = inv(model.F)*(expm(model.F*dt)- eye(size(model.F,1)))*model.B;
end

model.H = modelIn.H;
model.H(Nstates) = 1;
model.H(Nstates-1) = 0;              % Measurement matrix.

model.Q = modelIn.Q;
model.Q(Nstates, Nstates) = 0.00001; % Process noise covariance
model.R = modelIn.R;                 % Measurement noise covariance

% Checking observability
if ~(rank(obsv(model.F,model.H))==size(model.F,1))
    disp('System is not observable with only cgm measurements')
end

% Initial vectors
model.initCov = modelIn.initCov;
model.initCov(Nstates, Nstates)= 1;
model.x0 = modelIn.x0;
model.x0(Nstates) = 0;

% State names
model.stateNames = modelIn.stateNames;
model.stateNames{Nstates} = 'yLPF';

end

