% Function for calulcating accuracy measures 
% mean squared error and mean absolute percentage error of
% an estimate

% Important: t_hat must be bigger than t, in order for 
% the function to run correct 

function error = computeError(y, y_hat, t, t_hat)

% Storing
MAE = nan(1, length(t));
MSE = nan(1, length(t));
MAPE = nan(1, length(t));
idx = nan(1, length(t));

for i=1:length(t)
    [~,j] = min(abs(t_hat-t(i))); % Find the closest point in time
    idx(i) = j;                   % Index of the closes point in time to t(i)
    e = y(i)-y_hat(j);            % Residual
    MAE(i) = abs(e);
    MSE(i) = e^2;
    MAPE(i) =((abs(e))/y(i))*100;
end
MAE(isnan(MAE))=0;
MSE(isnan(MSE))=0;
MAPE(isnan(MAPE))=0;

n = length(t);
%error.MAE = (1/n)*sum(MAE);    % Mean absolute error (not used in master thesis)
error.MSE = (1/n)*sum(MSE);     % Mean squared error
%error.RMSE = sqrt(error.MSE);  % Root mean squared error (not used in master thesis)
error.MAPE = (1/n)*sum(MAPE);   % Mean absolute percentage error
end
