% Linear regression with optional weigheted least squares- and 
% intercept solution

function y = linearRegression(x,Y, intercept, weighted)
if intercept 
    if weighted
        X = [ones(size(x,1), 1) x];
        W = getWeights(x);
        b = inv(X'*W*X)*X'*W*Y;
        y =X*b;
    else
        X = [ones(size(x,1), 1) x];
        b = inv(X'*X)*X'*Y;
        y =X*b;
    end
    
else
    if weighted 
        W = getWeights(x);
        b = inv(x'*W*x)*x'*W*Y;
        y =x*b;
    else
        b = inv(x'*x)*x'*Y;
        y =x*b;
    end
    
end
end

%%% Helper methods %%%
% Helper method for calculating wights using tricube weight function used, 
% William S. Cleveland (1979) 
function W = getWeights(x)
x0 = x(1);
temp = abs(x0-x);
d = 1;
w = (1-abs((x0-x)/d).^3).^3;
W = diag(w);
end

