clc
clear all
close all

%% Data 
% date_var1=435; % weigth = 40 kg
% csv1 = 'data\Appel_sensorData_10-03-2021.csv';
% xlsx1 = 'data\Data_storage_identif_10.03.21.xlsx';

date_var2=430; % weigth = 36 g
csv2 = 'data\Appel_sensorData_05-03-2021.csv';
xlsx2 = 'data\Data_storage_identif_05.03.21.xlsx';

% date_var3=426; % weigth = 36 kg
% csv3 = 'data\Appel_sensorData_01-03-2021.csv';
% xlsx3 = 'data\Data_storage_identif_01.03.21.xlsx';
 
% date_var4=325; 
% csv4 = 'data\Appel_sensorData_20-11-2020.csv';
% xlsx4 = 'data\Data_storage_identif_20.11.20.xlsx';
 
% date_var5=323; 
% csv5 = 'data\Appel_sensorData_18-11-2020.csv';
% xlsx5 = 'data\Data_storage_identif_18.11.20.xlsx';


%%% Note:
% csvX, xlsxX and date_varX must correspond to correct, requested data
% files.

%%% CGM data
[t, y_cgm, ~] = getCGMdata(csv2);

%%% BGA data
[t_BGA, y_BGA, ~, ~, ~,~, ~, ~] = getBGAdata(xlsx2, date_var2);

%% Convert to relative time
tStart = t(1);
t_BGA_rel=convertToRelativeTime(t_BGA, tStart);
t = convertToRelativeTime(t, tStart);

%% Model parameters
Tisf = 6;       % Time constant for diffusion from blood plasma to ISF [min]
a    = -0.0018; % Rate of decay [-]
b    = 0.004;   % Smoothing factor for low pass filter [-]
dt   = 1.2/60;  % Sampling rate [min]

%% Choose method (1->Do, 0->Don't do)
do_measurement_comparison = 1;
do_kf                     = 1;
do_mest                   = 1;
do_rtskf                  = 1;
do_FD                     = 1;

%% Compare BGA and CGM 
if do_measurement_comparison 
    
    error_measurement = computeError(y_BGA, y_cgm, t_BGA_rel, t);

    figure(1)
    plot(t, y_cgm, 'LineWidth',1.5,'DisplayName', 'CGM')
    hold on
    plot(t_BGA_rel, y_BGA, 'k.','MarkerSize',15,'DisplayName', 'BGA')
    legend('show', 'location', 'best')
    hold off
    grid on
    title('CGM and BGA Measurements')
    xlim([t(1) t(end)])
    xlabel('Time [min]')
    ylabel('Glucose [mmol/L]')
    
    disp('Measurement comparison done!')
end

%% BG level estimation - KF
if do_kf
    
    % Filter output and accuracy scores
    [kf_output, kf_error] = standardKF(y_cgm, t, Tisf, a, b, dt, y_BGA, t_BGA_rel,... 
                           'plotResult',1,'useRegression',1,...
                           'useLowPassFilter', 0,...
                           'useMAsmoothing', 1);
    
    disp('KF reconstruction of BG done!')
end

%% BG level estimation - M-Robust KF
if do_mest
    
    % Filter output and accuracy scores
    [mestkf_output, mestkf_error] = MrobustKF(y_cgm, t, Tisf, a, b, dt, y_BGA, t_BGA_rel,... 
                           'plotResult',1,'useRegression',1,...
                           'useLowPassFilter', 0,...
                           'useMAsmoothing', 1);

    
    disp('Mest-KF reconstruction of BG done!')
end

%% BG level estimation - RTSKF-UI
if do_rtskf
    
    % Filter output and accuracy scores
    [rtskf_output, rtskf_error] = robustTwoStageKFui(y_cgm, t, Tisf, a, dt, t_BGA_rel, y_BGA,...
                    'plotResult',1);
   
    disp('RTSKF-UI reconstruction of BG done!')

end

%% BG level estimation - FD approximation
if do_FD
    
    % Filter output and accuracy scores
    [FD_output, FD_error] = FDapprox(y_cgm, Tisf, dt, t, y_BGA, t_BGA_rel,'plotResult',1,'useRegression',1, 'useMAsmoothing',1);
    
    disp('FD approximation of BG done!')
end

%%



      