%% Get data
clc
tic
clear all
close all

%% Data 
date_var1=435; % weigth = 40 kg
csv1 = 'data\Appel_sensorData_10-03-2021.csv';
xlsx1 = 'data\Data_storage_identif_10.03.21.xlsx';

date_var2=430; % weigth = 36 g
csv2 = 'data\Appel_sensorData_05-03-2021.csv';
xlsx2 = 'data\Data_storage_identif_05.03.21.xlsx';

date_var3=426; % weigth = 36 kg
csv3 = 'data\Appel_sensorData_01-03-2021.csv';
xlsx3 = 'data\Data_storage_identif_01.03.21.xlsx';

date_var4=325; 
csv4 = 'data\Appel_sensorData_20-11-2020.csv';
xlsx4 = 'data\Data_storage_identif_20.11.20.xlsx';

date_var5=323; 
csv5 = 'data\Appel_sensorData_18-11-2020.csv';
xlsx5 = 'data\Data_storage_identif_18.11.20.xlsx';

% CGM data
[t1, y_cgm11, y_cgm12] = getCGMdata(csv1);
[t2, y_cgm21, y_cgm22] = getCGMdata(csv2);
[t3, y_cgm31, y_cgm32] = getCGMdata(csv3);
[t4, y_cgm41, y_cgm42] = getCGMdata(csv4);
[t5, y_cgm51, y_cgm52] = getCGMdata(csv5);

% BGA data
[t_bga1, ybga1] = getBGAdata(xlsx1, date_var1);
[t_bga2, ybga2] = getBGAdata(xlsx2, date_var2);
[t_bga3, ybga3] = getBGAdata(xlsx3, date_var3);
[t_bga4, ybga4] = getBGAdata(xlsx4, date_var4);
[t_bga5, ybga5] = getBGAdata(xlsx5, date_var5);

%% Fix time
tStart1 = t1(1);
t_bga_rel1=convertToRelativeTime(t_bga1, tStart1);
t_rel1=convertToRelativeTime(t1, tStart1);

tStart2 = t2(1);
t_bga_rel2=convertToRelativeTime(t_bga2, tStart2);
t_rel2=convertToRelativeTime(t2, tStart2);

tStart3 = t3(1);
t_bga_rel3=convertToRelativeTime(t_bga3, tStart3);
t_rel3=convertToRelativeTime(t3, tStart3);

tStart4 = t4(1);
t_bga_rel4=convertToRelativeTime(t_bga4, tStart4);
t_rel4=convertToRelativeTime(t4, tStart4);

tStart5 = t5(1);
t_bga_rel5=convertToRelativeTime(t_bga5, tStart5);
t_rel5=convertToRelativeTime(t5, tStart5);

%% Prepare data
[ycgm1, ~] = getEqualLenVec(y_cgm11, t_rel1, t_bga_rel1);
[ycgm2, ~] = getEqualLenVec(y_cgm21, t_rel2, t_bga_rel2);
[ycgm3, ~] = getEqualLenVec(y_cgm31, t_rel3, t_bga_rel3);
[ycgm4, ~] = getEqualLenVec(y_cgm41, t_rel4, t_bga_rel4);
[ycgm5, ~] = getEqualLenVec(y_cgm51, t_rel5, t_bga_rel5);

%% Identify T
% T = 10 [min] usually reported in litterature
% hence it is set as the starting point in the optizimzation process
initT = 10;

options = optimset('PlotFcns',@optimplotfval);
fun1 = @(T)objFuncT(ycgm1, ybga1, T);
T1 = fminsearch(fun1, initT); %, options);

fun2 = @(T)objFuncT(ycgm2, ybga2, T);
T2 = fminsearch(fun2, initT); %, options); 

fun3 = @(T)objFuncT(ycgm3, ybga3, T);
T3 = fminsearch(fun3, initT); %, options); 

fun4 = @(T)objFuncT(ycgm4, ybga4, T);
T4 = fminsearch(fun4, initT); %, options); 

fun5 = @(T)objFuncT(ycgm5, ybga5, T);
T5 = fminsearch(fun5, initT); %, options); 

T = [T1;T2;T3;T4;T5];
%% Identify a
% a = -0.05 in Staal et. al (2018)
% hence it is the starting point of this optimzation process
inita = -0.05;

% Calculate derivatives
dt = 5; % Sampling time
[ydot1, yddot1] = getDerivatives(ybga1,dt);
[ydot2, yddot2] = getDerivatives(ybga2,dt);
[ydot3, yddot3] = getDerivatives(ybga3,dt);
[ydot4, yddot4] = getDerivatives(ybga4,dt);
[ydot5, yddot5] = getDerivatives(ybga5,dt);

% Find a
fun1 = @(a)objFunca(yddot1, ydot1, a);
a1 = fminsearch(fun1, inita); %, options);

fun2 = @(a)objFunca(yddot2, ydot2, a);
a2 = fminsearch(fun2, inita); %, options);

fun3 = @(a)objFunca(yddot3, ydot3, a);
a3 = fminsearch(fun3, inita); %, options);

fun4 = @(a)objFunca(yddot4, ydot4, a);
a4 = fminsearch(fun4, inita); %, options);

fun5 = @(a)objFunca(yddot5, ydot5, a);
a5 = fminsearch(fun5, inita); %, options);

a = [a1;a2;a3;a4;a5];
mean_a = mean(a);
%% Helper methods %%

% Objective function for identification of T
function rss = objFuncT(y,u,T)
yhat = diffusionEq(u,T);
rss = sum((y-yhat).^2);
end

% Objective function for identification of a
function rss = objFunca(y,u,a)
yhat = derivativeEq(u,a);
yhat = yhat(1:end-1)';
rss = sum((y-yhat).^2);
end

% Gp to Gisf 1.order diffusion differential equation
function yhat = diffusionEq(u,T)
dt = 5;
yhat(1) = 0;
for i=1:length(u)-1    
    dy = (1/T)*(u(i)-yhat(i));
    yhat(i+1) = yhat(i) + dy*dt;
end
yhat = yhat';
end

% Get first and second derivative
function [ydot, yddot] = getDerivatives(y,dt)
for i=1:length(y)-1
    ydot(i) = (y(i+1)-y(i))/dt;
end

for i=1:length(ydot)-1
    yddot(i) = (ydot(i+1)-ydot(i))/dt;
end
end

% Gp dynamic of second derivative differential equation 
function yhat = derivativeEq(u,a)
dt = 5;
yhat(1) = 0;
for i=1:length(u)-1    
    dy = a*u(i);
    yhat(i+1) = yhat(i) + dy*dt;
end
yhat = yhat';
end





