% Helper method for constructing system models

function model = setModel(T,dt,a,modelNum)
if modelNum == 1 %%% Step disturbance model %%% (not used in master thesis)
    
    model.F =[1 0; 1/T -1/T];                   % System matrix (continuous); 
    model.Phi=expm(model.F*dt);                 % System matrix (discrete)
    model.H=[0 1];                              % Measurement matrix 
    model.Q = [5000 0; 0 1];                    % Process noise covariance matrix.
    model.R =1;                                 % Measurement noise covariance
    
    % Checking observability
    if ~rank(obsv(model.F,model.H))==size(model.F,1)  
        disp('System is not observable with only cgm measurements')
    end
    
    % Initial vectors
    model.initCov = diag([1 1]);
    model.x0 = [0; 0];
    
    % Discretization
    model.Phi=expm(model.F*dt);
    
    % State names
    model.stateNames = {'Gp', 'Gisf'}; 

elseif modelNum == 2 %%% Ramp disturbance model %%% (not used in master thesis)

    model.F=[1 1 0;0 1 0; 1/T 0 -1/T];          % System matrix (continuous)
    model.Phi=expm(model.F*dt);                 % System matrix (discrete)
    model.H=[0 0 1];                            % Measurement matrix.
    model.Q=[0.001 0 0;0 400 0; 0 0 0.001];     % Process noise covariance
    model.R=0.000001;                           % Measurement noise covariance
    
    % Checking observability
    if ~(rank(obsv(model.F,model.H))==size(model.F,1))
        disp('System is not observable with only cgm measurements')
    end
    
    % Initial vectors
    model.initCov = diag([1 1 1]);
    model.x0 = [0;0;0];
    
    % State names
    model.stateNames = {'Gp','dGp','Gisf'}; 
    
elseif modelNum == 3 %%% Rate-only model %%%
    
    model.F=[0 1 0;0 a 0; 1/T 0 -1/T];           % System matrix (continuous)
    model.Phi=expm(model.F*dt);                  % System matrix (discrete)
    model.H=[0 0 1];                             % Measurement matrix. 
    model.Q=[0.001 0 0;0 0.00007 0; 0 0 0.001];  % Process noise covariance 
    model.R=0.85;                                % Measurement noise covariance
    
    % Checking observability
    if ~(rank(obsv(model.F,model.H))==size(model.F,1)) 
        disp('System is not observable with only cgm measurements')
    end
    
    % Initial covariance
    model.initCov = diag([1 1 1]);
    model.x0 = [0;0;0];

    % State names
    model.stateNames = {'Gp','dGp','Gisf'}; 
    
elseif modelNum == 4 %%% Steil-Rebrin with UI %%%
    model.F = -1/T;                                                            % System matrix (continuous)
    model.E = 1/T;                                                             % Unknown input transition matrix (continuous)
    model.Phi= expm(model.F*dt);                                               % System matrix (discrete)
    model.Eps = inv(model.F)*(expm(model.F*dt)- eye(size(model.F,1)))*model.E; % Unknown input transition matrix (discrete)
    model.H = 1;                                                               % Measurement matrix. 
    model.Q = 0.001;                                                           % Process noise covariance
    model.R = 0.1;                                                             % Measurement noise covariance 

    % Checking observability
    if ~(rank(obsv(model.F,model.H))==size(model.F,1)) 
        disp('System is not observable with only cgm measurements')
    end

    % Initial vectors
    model.initCov = 1;   
    model.x0 = 0; 
    
    % State/input names
    model.stateNames = {'Gisf'};
    model.inputNames = {'Gp'};
end

end

